package blueeyes.io.iteratee

sealed trait IterateeErrorRemedy
case object MoreInput  extends IterateeErrorRemedy
case object RetryChunk extends IterateeErrorRemedy
case object NoRemedy   extends IterateeErrorRemedy

sealed abstract class IterateeError(message: String, cause: Option[Throwable]) extends Exception(message, cause.orNull) {
  def remedy: IterateeErrorRemedy
  
  def withRemedy(r: IterateeErrorRemedy) = new IterateeError(message, cause) {
    def remedy = r
  }
  
  def withoutRemedy = new IterateeError(message, cause) {
    def remedy = NoRemedy
  }
}

class InadequateInput(message: String, cause: Option[Throwable]) extends IterateeError(message, cause) {
  def remedy = MoreInput
}

object InadequateInput {
  def apply(message: String = "Expected more input", cause: Option[Throwable] = None) = new InadequateInput(message, cause)
}

class MalformedInput(message: String, cause: Option[Throwable]) extends IterateeError(message, cause) {
  def remedy = RetryChunk
}

object MalformedInput {
  def apply(message: String = "Malformed input", cause: Option[Throwable] = None) = new MalformedInput(message, cause)
}

object AlreadyDone extends IterateeError("The iteratee already finished and cannot produce more output", None) {
  def remedy = NoRemedy
}

class InternalError(message: String, cause: Option[Throwable], val remedy: IterateeErrorRemedy) extends IterateeError(message, cause)

object InternalError {
  def apply(message: String, cause: Option[Throwable] = None, remedy: IterateeErrorRemedy = NoRemedy) = new InternalError(message, cause, remedy)
}