package blueeyes.io.iteratee

/** A typeclass for a source of data that is vector-like.
 */
trait VectorLike[L[_]] {
  def headOption[A](l: L[A]): Option[A] = if (len(l) > 0) Some(head(l)) else None
  
  def head[A](l: L[A]): A = index(l, 0)
  
  def tailOption[A](l: L[A]): Option[L[A]] = if (len(l) > 0) Some(tail(l)) else None
  
  def tail[A](l: L[A]): L[A] = slice(l, 1, len(l))
  
  def countWhile[A](l: L[A], f: A => Boolean): Int = {
    val length = len(l)
    var count = 0
    
    for (i <- 0 to length) {
      if (!f(index(l, i))) return count
      else count = count + 1
    }
    
    count
  }
  
  def take[A](l: L[A], n: Int) = { val length: Int = len(l); slice(l, 0, if (n > length) length else n) }
  
  def takeWhile[A](l: L[A], f: A => Boolean) = take(l, countWhile(l, f))
  
  def drop[A](l: L[A], n: Int) = { val length: Int = len(l); slice(l, if (n > length) length else n, length) }
  
  def dropWhile[A](l: L[A], f: A => Boolean) = drop(l, countWhile(l, f))
    
  def each[A](l: L[A], f: A => Unit): Unit = for (i <- 0 to len(l)) f(index(l, i))
  
  def splitAt[A](l: L[A], idx: Int): (L[A], L[A]) = (slice(l, 0, idx), slice(l, idx, len(l)))
  
  def foldLeft[A, Z](l: L[A], initial: Z)(f: (Z, A) => Z): Z = {
    var state = initial
    val length = len(l)
    
    for (i <- 0 to length) {
      state = f(state, index(l, i))
    }
    
    state
  }
  
  def isEmpty[A](l: L[A]): Boolean = len(l) == 0
  
  def flatMap[A, B](l: L[A], f: A => L[B]): L[B] = bind[A, B](l, f)
  
  def map[A, B](l: L[A], f: A => B): L[B] = flatMap(l, (e: A) => pure(f(e)))
  
  def len[A](l: L[A]): Int
  
  def index[A](l: L[A], idx: Int): A
  
  def pure[A](a: A): L[A]
  
  def bind[A, B](l: L[A], f: A => L[B]): L[B]
  
  def slice[A](l: L[A], from: Int, until: Int): L[A]
}

object VectorLike {
  
}